      MODULE OC_cc

      USE CMP_COMM, ONLY:

     >   MPI_COMM_Ocean => COMM_local,

     >   Coupler_id,
     >   component_master_rank_local,
     >   process_rank_local,
     >   component_nprocs,
     >   ibuffer,

     >   MPI_INTEGER,MPI_STATUS_SIZE,

     >   kind_REAL,kind_alt_REAL,MPI_kind_REAL,MPI_kind_alt_REAL

      implicit none

      real dtc            !<- Coupling period

      integer i_dtc2dto /100/,  !<- Coupling period / OM time step
     >        n_ts /-1/   !<- number of time step - 1
                          ! (increments/used in OC_SENDSST)

      integer imt,jmt,NX,NY,NLEV,nt,NGP,NGPsub,rc

      integer i_f,j_f,i_l,j_l

      integer ifc_g,jfc_g,ilc_g,jlc_g

      integer kind_sfcflux,kind_SST,kind_SLM,kind_Pe_SEND
      parameter (kind_sfcflux=kind_REAL,
     >           kind_SST=kind_REAL,
     >           kind_SLM=kind_REAL)
      parameter (kind_Pe_SEND=kind_REAL)
      integer MPI_DATATYPE_sfcflux

      integer, parameter :: nc=1  ! # of copies of each flux

      real,allocatable:: ALONt(:,:),ALATt(:,:),
     >                   ALONu(:,:),ALATu(:,:),
     >                   ALONv(:,:),ALATv(:,:)

c     real(kind=kind_SLM),allocatable:: SLMt(:,:),SLMu(:,:),SLMv(:,:)
!<- these are not needed other than for printout purposes. Uncomment
!if the latter require (note same indication elsewhere)

Controls:
      integer nunit_announce /6/, VerbLev /3/

      SAVE

      END MODULE OC_cc
C
C***********************************************************************
C
      MODULE SFLUX_cc

      USE OC_cc, ONLY: kind_sfcflux

      implicit none

      integer num_sflx /8/  ! # of surf. fluxes to be dealt with

      real(kind=kind_sfcflux),allocatable:: sflx(:,:,:)

      real missing_value_flag /-1.E30/ ! see subr-s OC_INIT, OC_RECV_SBC

      SAVE

      END MODULE SFLUX_cc
C
C***********************************************************************
C
      SUBROUTINE OC_CMP_START

      USE OC_cc, ONLY: process_rank_local,VerbLev,ibuffer,Coupler_id

      implicit none

      integer Ocean_id /2/, Ocean_master_rank_local /0/, Ocean_spec/2/
      integer ibuf(1)
      character*20 s
C

                      !<-id of OM as a component of the coupled system
      call CMP_INIT(Ocean_id,1)
                             !<-"flexibility level"
      if (Coupler_id.ge.0) VerbLev=min(VerbLev,ibuffer(4))

c     if (process_rank_local.eq.Ocean_master_rank_local) then
c       print*,'**OM: back from CMP_INIT, to call CMP_INTRO'
c     end if

      call CMP_INTRO(Ocean_master_rank_local)

      write(s,'(i2)') VerbLev
      call OC_ANNOUNCE('back from CMP_INTRO, VerbLev='//s,2)

      ibuf(1)=Ocean_spec

      call CMP_INTEGER_SEND(ibuf,1)

      write(s,'(i2)') ibuf(1)
      call OC_ANNOUNCE('OC_CMP_START: returning, sent Ocean_spec='//s,2)

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_INIT(imt_,jmt_,km_, i_f_, i_l_, j_f_,    j_l_,
                           !<-"total"     !<-"first"           !<-"last"
     >ifc_g_,ilc_g_,jfc_g_,jlc_g_,missing_value_flag_)
        !<-"comp."      !<-"global"
                     ! i_f_, i_l_, j_f_, j_l_ are local in tile
      USE OC_cc

      USE SFLUX_cc

      implicit none

      integer imt_,jmt_,km_,i_f_,i_l_,j_f_,j_l_,
     >ifc_g_,jfc_g_,ilc_g_,jlc_g_

      real missing_value_flag_
C

      call OC_ANNOUNCE('OC_INIT: entered',2)

      imt=imt_
      jmt=jmt_
      NX=imt_
      NY=jmt_
      NLEV=km_

      i_f=i_f_
      i_l=i_l_
      j_f=j_f_
      j_l=j_l_

      ifc_g=ifc_g_
      jfc_g=jfc_g_
      ilc_g=ilc_g_
      jlc_g=jlc_g_

      NGPsub=(i_l-i_f+1)*(j_l-j_f+1)

      NGP=NY*NX

c     if(process_rank_local.eq.component_master_rank_local) then
        allocate(ALONt(NX,NY),ALATt(NX,NY),
     >  ALONu(NX,NY),ALATu(NX,NY),
     >  ALONv(NX,NY),ALATv(NX,NY))

c       allocate(SLMt(NX,NY),
c    >  SLMu(NX,NY),SLMv(NX,NY))
!<- these are not needed other than for printout purposes. Uncomment
!if the latter require (note same indication elsewhere)
c     end if
!<- if/endif commented out in case assembling/dissasembling routines
! somehow need these arrays allocated for all processes

      allocate(sflx(i_f:i_l,j_f:j_l,num_sflx))

      if (kind_sfcflux.eq.kind_REAL) then
        MPI_DATATYPE_sfcflux=MPI_kind_REAL
      else if (kind_sfcflux.eq.kind_alt_REAL) then
        MPI_DATATYPE_sfcflux=MPI_kind_alt_REAL
      else
        call GLOB_ABORT(1,
     >  'kind_sfcflux is neither kind_REAL nor kind_alt_REAL',1)
      end if

      missing_value_flag_=missing_value_flag

      if (VerbLev.ge.3) print*,'OC_INIT: returning ',
     >imt_,jmt_,km_,i_f_,i_l_,j_f_,j_l_,ifc_g_,ilc_g_,jfc_g_,jlc_g_

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_SENDGRIDS(xt,yt,xu,yu,xv,yv,dt)

      USE OC_cc

      implicit none

      real(kind=kind_REAL), dimension(i_f:i_l,j_f:j_l) ::
     >xt,yt,xu,yu,xv,yv
      real(kind=kind_REAL) dt
      
      integer ibuf(2)
      real buf(1)
      character*20 s
C

      if (Coupler_id.lt.0) return    !   <- standalone mode

      buf(1)=dt   ! <- OM time step
      write(s,'(1pe20.12)') dt
      call OC_ANNOUNCE('(BP) to send time step='//s,2)
      call CMP_SEND(buf,1)
      call OC_ANNOUNCE('(BP) time step='//s//' sent',1)

      IF (component_master_rank_local.eq.process_rank_local) THEN

        call CMP_RECV(buf,1)
        dtc=buf(1)
        write(s,'(1pe20.12)') dtc
        call OC_ANNOUNCE('coupling period received: '//s,1)
        i_dtc2dto=nint(dtc/dt)
        if (abs(i_dtc2dto-dtc/dt).gt.1.E-7) call GLOB_ABORT(1,
     >  'OM: ABORTED: dtc is not a multiple of dt',1)

        ibuf(1)=NX
        ibuf(2)=NY
        call OC_ANNOUNCE('to send grid dimensions',2)
        call CMP_INTEGER_SEND(ibuf,2)
        call OC_ANNOUNCE('grid dimensions sent',1)

        ibuf(1)=i_dtc2dto

           print*,'xu(1:3,1:3): '
           print*,xu(1:3,1:3)
           print*,'yu(1:3,1:3): '
           print*,yu(1:3,1:3)

      END IF

      call MPI_BCAST(ibuf,1,MPI_INTEGER,component_master_rank_local,
     >MPI_COMM_Ocean,rc)
      i_dtc2dto=ibuf(1)

      call OC_ANNOUNCE('(BP) i_dtc2dto broadcast OK',2)

      call ASSEMBLE_cc(ALONt,xt)
      call ASSEMBLE_cc(ALATt,yt)
      call ASSEMBLE_cc(ALONu,xu)
      call ASSEMBLE_cc(ALATu,yu)
      call ASSEMBLE_cc(ALONv,xv)
      call ASSEMBLE_cc(ALATv,yv)

         IF (component_master_rank_local.eq.process_rank_local) THEN
           print*,'xu(1:3,1:3): '
           print*,xu(1:3,1:3)
           print*,'yu(1:3,1:3): '
           print*,yu(1:3,1:3)

           print*,'ALONu(1:3,1:3): '
           print*,ALONu(1:3,1:3)
           print*,'ALATu(1:3,1:3): '
           print*,ALATu(1:3,1:3)
         END IF


      call OC_ANNOUNCE('(BP) to send grid arrays (6 MPI calls)',2)

      call CMP_SEND(ALONt,NGP)
      call CMP_SEND(ALATt,NGP)
      call CMP_SEND(ALONu,NGP)
      call CMP_SEND(ALATu,NGP)
      call CMP_SEND(ALONv,NGP)
      call CMP_SEND(ALATv,NGP)

      call OC_ANNOUNCE('the 6 grid arrays sent',1)

      call OC_ANNOUNCE('(BP) OC_SENDGRIDS: returning',2)

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_ANNOUNCE(s,DbgLev)

      USE OC_cc, ONLY: nunit_announce,VerbLev,MPI_COMM_Ocean

      implicit none

      character*(*) s
      integer DbgLev

      integer ierr
C
      if (DbgLev.le.VerbLev) then
        if (s(1:5).eq.'(BP) ') then
          call MPI_BARRIER(MPI_COMM_Ocean,ierr)
        end if
        CALL CMP_ANNOUNCE(nunit_announce,'OM: '//s)
      end if

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_SENDSLM(mt,mu,mv)

      USE OC_cc

      implicit none

      integer, dimension(i_f:i_l,j_f:j_l) :: mt,mu,mv

      real fl(i_f:i_l,j_f:j_l),FG(NX,NY) !type/kind: see OC_RECVSBC
      integer j,i
C

      if (Coupler_id.lt.0) return    !   <- standalone mode

C       mt, mu, mv are supposed to be 1 at sea and 0 on land

      fl=mt
      call ASSEMBLE_cc(FG,fl)
      call CMP_SEND(FG,NGP)
      fl=mu
      call ASSEMBLE_cc(FG,fl)
      call CMP_SEND(FG,NGP)
      fl=mv
      call ASSEMBLE_cc(FG,fl)
      call CMP_SEND(FG,NGP)

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_SENDSST(temp,ntl)

      USE OC_cc

      implicit none

      real temp(i_f:i_l,j_f:j_l,NLEV,2) ! corresponding actual arg. is
                                        ! declared as "real" in HYCOM
                                        ! (common_blocks.h). Therefore,
                                        ! this routine must be compiled
                                        ! in the same way as
                                        ! common_blocks.h so that "real"
                                        ! may mean the same
      integer ntl  ! "num. of time lev.", see code, must be 1 or 2"
      
      real sst(i_f:i_l,j_f:j_l) !corresponding dummy arg. in ASSEMBLE_cc
                                !is "real", since corresponding dummy
                                !arg. in xcaget is "real"
      real SSTG(NX,NY) ! corresponding dummy arg. in ASSEMBLE_cc
                           ! is "real", since corresponding dummy arg.
                           ! in xcaput is "real". If type/kind differs
                           ! from that of the form. arg. of CMP_SEND,
                           ! another array must be declared/used in
                           ! call CMP_SEND
      character*20 s
C

      n_ts=n_ts+1

      if (VerbLev.ge.3) write(s,'(2i10)') n_ts,i_dtc2dto
      call OC_ANNOUNCE('OC_SENDSST: n_ts, i_dtc2dto'//s,3)

      if (Coupler_id.lt.0) RETURN  !   <- standalone mode

      IF ((n_ts/i_dtc2dto)*i_dtc2dto.ne.n_ts) RETURN

      if (ntl.lt.1 .or. ntl.gt.2) then
        call GLOB_ABORT(1,'** OC_SENDSST: ntl != 1 or 2 **',1)
      end if

      if (NX.ne.imt .or. NY.ne.jmt) then
        call GLOB_ABORT(1,
     >  '** OC_RECVSBC: NX.ne.imt.or.NY.ne.jmt',1)
      end if
             ! NX, NY are sizes of array for which interpolated
             ! values are computed by C. imt, jmt are sizes of whole
             ! grid array in HYCOM. If the two pairs are not the same
             ! (normally, it can be that imt>NX and/or jmt>NY
             ! because of boundaries), new array SSTG1(imt,jmt) needs to
             ! be declared and used in call ASSEMBLE_cc, then SSTG be
             ! defined as SSTG=SSTG1(x:x+NX-1,y:y+NY-1) and used
             ! call CMP_SEND

      sst=temp(:,:,1,ntl)
      call OC_ANNOUNCE('OC_SENDSST: to call ASSEMBLE_cc(SSTG,sst)',3)

      call ASSEMBLE_cc(SSTG,sst)

      call CMP_SEND(SSTG,NGP)

      call OC_ANNOUNCE('OC_SENDSST: to return',3)

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_RECV_SBC(SF)

      USE OC_cc

      USE SFLUX_cc

      implicit none

      real SF(i_f:i_l,j_f:j_l,nc)     ! corresponding actual args are
                                      ! declared as "real" in HYCOM
                                      ! (common_blocks.h). Therefore,
                                      ! this routine must be compiled in
                                      ! the same way as common_blocks.h
                                      ! so that "real" may mean the same
                                      ! (see also below about F)
                                           
      real F(NX,NY) ! corresponding dummy arg. in DISASSEMBLE_cc
                        ! is "real", since corresponding dummy arg.
                        ! in xcaput is "real". If the type/kind differs
                        ! from that of the form. arg. of CMP_RECV,
                        ! another array must be declared/used in
                        ! call CMP_RECV
      real fl(i_f:i_l,j_f:j_l) !corresponding dummy a. in DISASSEMBLE_cc
                               !is "real", since corresponding dummy
                               !arg. in xcaput is "real"
      real too_low /-1.E20/ ! must be >= very_large_negative in
                            ! interp. routine, currently -1.E30

      integer i,j,m,n,ierr
      integer nts,n_ts_old/-100/
      logical SENDSSTcomesFIRST/.true./
      save too_low,n_ts_old,SENDSSTcomesFIRST,n
C

      call OC_ANNOUNCE('OC_RECV_SBC entered',3)
c        print*,'OC_RECV_SBC: entered'

      if (NX.ne.imt .or. NY.ne.jmt) then
        call GLOB_ABORT(1,'** OC_RECVSBC: NX.ne.imt.or.NY.ne.jmt',1)
      end if
             ! NX, NY are sizes of array for which interpolated
             ! values are computed by C. imt, jmt are sizes of whole
             ! grid array in HYCOM. If the two pairs are not the same
             ! (normally, it can be that imt>NX and/or jmt>NY
             ! because of boundaries), new array F1(imt,jmt) needs to
             ! be declared, then defined as F1(x:x+NX-1,y+NY-1)=F,
             ! then boundary values (1:x-1,x+NX:imt,1:y-1,y+NY:jmt)
             ! be defined for F1, then F1 (rather than F) be used in
             ! DISASSEMBLE_cc

      if (n_ts.eq.-1) SENDSSTcomesFIRST=.false.
      if (SENDSSTcomesFIRST) then
        nts=n_ts
      else
        nts=n_ts+1
      end if

      if (n_ts.eq.n_ts_old) then
        n=n+1
      else
        n=1
        n_ts_old=n_ts
      end if
c dong initilize fl as 0.0
      fl = 0.0

      call GLOB_ABORT(max(n-num_sflx,0),
     >'number of surf. flux exceeds num_sflx in OC_RECVSBC',1)

      if (Coupler_id.lt.0) return     !   <- standalone mode

      IF ((nts/i_dtc2dto)*i_dtc2dto.eq.nts) THEN

        call CMP_RECV(F,NGP)
c        if (process_rank_local .eq. 0) print *,'dong F',F

        call DISASSEMBLE_cc(F,fl)

        sflx(:,:,n)=fl

      ELSE

        fl=sflx(:,:,n)

      END IF

c        print *,'dong too_low',process_rank_local,too_low
c        do j=j_f,j_l
c          do i=i_f,i_l
c           print *,'dong',i,j,fl(i,j)
c          end do
c        end do
                                   
      do m=1,nc
        do j=j_f,j_l
          do i=i_f,i_l
            if (fl(i,j).gt.too_low) then
              SF(i,j,m)=fl(i,j)
            else if (missing_value_flag.ne.0.) then
              SF(i,j,m)=missing_value_flag
            end if
          end do
        end do
      end do
c      print *,'dong too_low finish',process_rank_local
c      call MPI_BARRIER(MPI_COMM_Ocean,ierr)

      call OC_ANNOUNCE('OC_RECV_SBC to return',3)
c        print*,'OC_RECV_SBC: to return'

      return
      END
C
C***********************************************************************
C
      SUBROUTINE OC_DUMMY_SBC(precip,snsibl,latent,mslprs,radflx,swflx,
     >taux,tauy,mt,mu,mv)
      
      USE OC_cc

      USE CMP_COMM, ONLY: nprocs_global

      implicit none

      real, dimension(i_f:i_l,j_f:j_l,nc) :: 
     >precip,snsibl,latent,mslprs,radflx,swflx,taux,tauy
      integer, dimension(i_f:i_l,j_f:j_l) :: mt,mu,mv

      integer i,j,k
      logical dlt,dlu,dlv
C

      IF (nprocs_global-component_nprocs.gt.1) THEN  !i.e. if not only
        call OC_ANNOUNCE('Dummy SBC off, AM, C present',3) !Coupler but
        RETURN                                       !also AM is present
      END IF

      IF (Coupler_id.ge.0) THEN              !i.e, if Coupler is present
        call OC_ANNOUNCE(
     >  'Dummy SBC off but compared with, C (but not AM) present',3)
        dlt=.false.
        dlu=.false.
        dlv=.false.
        jloop: do j=j_f,j_l
        do i=i_f,i_l
          do k=1,nc
            if (precip(i,j,k).ne.0.
     >      .or.snsibl(i,j,k).ne.200.
     >      .or.latent(i,j,k).ne.0.
     >      .or.mslprs(i,j,k).ne.101300.
     >      .or.radflx(i,j,k).ne.200.
     >      .or.swflx(i,j,k).ne.100.
     >      ) then
              if (mt(i,j).eq.1) then
                print*,'**Discrepancy at sea T-point:**',
     >          process_rank_local,i,j,
     >          precip(i,j,:),
     >          snsibl(i,j,:),
     >          latent(i,j,:),
     >          mslprs(i,j,:),
     >          radflx(i,j,:),
     >          swflx(i,j,:)
                exit jloop
              else
                dlt=.true.
              end if
            end if
            if (taux(i,j,k).ne.1.E-3) then
              if (mu(i,j).eq.1) then
                print*,'**Discrepancy at sea u-point:**',
     >          process_rank_local,i,j,
     >          taux(i,j,:)
                exit jloop
              else
                dlu=.true.
              end if
            end if
            if (tauy(i,j,k).ne.1.E-2) then
              if (mv(i,j).eq.1) then
                print*,'**Discrepancy at sea v-point:**',
     >          process_rank_local,i,j,
     >          tauy(i,j,:)
                exit jloop
              else
                dlv=.true.
              end if
            end if
          end do
        end do
        end do jloop
        if (dlt) print*,'There are differences at T non-sea points',
     >  component_master_rank_local
        if (dlu) print*,'There are differences at u non-sea points',
     >  component_master_rank_local
        if (dlv) print*,'There are differences at v non-sea points',
     >  component_master_rank_local
        RETURN
      END IF

      call OC_ANNOUNCE(
     >'Dummy SBC on, OM standalone',3)
      precip=0.
      snsibl=200.
      latent=0.
      mslprs=101300.
      radflx=200.
      swflx=100.
      taux=1.E-3
      tauy=1.E-2

      RETURN
      END
